/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <wx/utils.h>
#include "ClickIt.h"

#include <iostream>
#include <sstream>
using namespace std;

IMPLEMENT_APP(ClickApp)

bool ClickApp::OnInit()
{
  ClickFrame *clickFrame = new ClickFrame("ClickIt by Visti Andresen");
  SetTopWindow(clickFrame);
  clickFrame->Show();
  return true;
}


  
ClickFrame::ClickFrame(const wxString name)
    : wxFrame(0, -1, name)
{
  lineNumber_ = -1;
  
  loadFileDialog_ = new wxFileDialog(this, _("Choose a program to load"),
                      "","", "*", wxOPEN|wxCHANGE_DIR);
  
  saveFileDialog_= new wxFileDialog(this, _("Choose a file to save Program to"),
                      "","", "*", wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
  
  menuBar_ = new wxMenuBar(wxMB_DOCKABLE);
  menuFile_ = new wxMenu();
  menuFile_->Append(new wxMenuItem(menuFile_, MENU_LOAD, _("&Load"),
                    _("Load a program")));
  menuFile_->Append(new wxMenuItem(menuFile_, MENU_SAVE, _("&Save"),
                _("Save the program")));
  menuBar_->Append(menuFile_, _("&File"));
  
  
  SetMenuBar(menuBar_);
  
  wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL); 
    
  wxBoxSizer *controls = new wxBoxSizer(wxHORIZONTAL); 
    
  controls->Add(
  new wxTextCtrl(this, ENTER_BOX,"", wxDefaultPosition, wxDefaultSize,
			     wxTE_PROCESS_ENTER ), 0, wxLEFT, 5);
   
  running_ = new wxCheckBox (this, MODE, _("Running"));
  controls->Add(running_, 0, wxLEFT|wxRIGHT, 5);
  
  pause_ = new wxCheckBox(this, -1, _("Pause"));
  controls->Add(pause_, 0, wxRIGHT, 5);
  controls->Add(new wxButton(this, HELP_BUTTON, "Help"), 0 , wxALL, 5);
    
  sizer->Add(controls, 0, wxEXPAND|wxTOP|wxBOTTOM,5);    
  
  wxBoxSizer *data = new wxBoxSizer(wxHORIZONTAL); 
  lineNumberText_ = new wxStaticText(this, -1, "LineNumber: ?????");
  data->Add(lineNumberText_, 0, wxLEFT|wxRIGHT, 5);
  
  status_ = new wxStaticText(this, -1, "");
  data->Add(status_, 0, wxRIGHT, 5);


  sizer->Add(data, 0, wxEXPAND|wxBOTTOM,5);    
  
  
  text_ = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxSize(300,200),
                         wxTE_MULTILINE);
  sizer->Add(text_, 1, wxALL|wxEXPAND, 5);
    
  sizer->Fit(this);
//  sizer->SetSizeHints();
  SetSizer(sizer);
  

  display_ = XOpenDisplay(NULL);
  if (display_ == 0)
  {
    wxMessageBox(_("Couldn't open display!"));
    throw false;
  }

}

ClickFrame::~ClickFrame()
{
  delete loadFileDialog_;
  delete saveFileDialog_;  
}

void ClickFrame::onHelp()
{
  string message;
  message += "This is a very tiny help. But it's a simpel program.\n\n";
  message += "mdelay(int miliseconds)\n";
  message += "  pause for 'miliseconds' ms\n";
  message += "lclick(int x, int y, int miliseconds, bool restore)\n";
  message += "  make a left click at 'x','y' hold button for 'miliseconds' ms,";
  message +="\n  if 'restore' is 1 return mouse pointer to original position\n";
  message += "mclick(int x, int y, int miliseconds, bool restore)\n";
  message += "rclick(int x, int y, int miliseconds, bool restore)\n";
  message += "goto(labelName)\n";
  message += "  jump to 'labelName'\n";
  message += "gosub(labelName)\n";
  message += "  jump to 'labelName' storing the current line number on a stack\n";
  message += "return\n";
  message += "  returns to the line number stored on the stack\n";  
  message += "offset(int x, int y)\n";
  message += "  sets an offset that will be added to mouse click command ";
  message += "coordinates\n\n";
  
  message += "\"labelName:\" defines a label\n";
  message += "Use the text input to the left of the Help button to\n";
  message += "create lclick commands at the current mouse position\n";
  message += "(press enter in it)\n";
  message += "bool's can only be 1 or 0 not true or false";
  wxMessageBox(message.c_str());
}

void ClickFrame::onLoad()
{
  if (loadFileDialog_->ShowModal() == wxID_OK){
    wxString file = loadFileDialog_->GetPath();
    text_->LoadFile(file);
  }
}

void ClickFrame::onSave()
{
  if (saveFileDialog_->ShowModal() == wxID_OK){
    wxString file = saveFileDialog_->GetPath();
    text_->SaveFile(file);
  }
}

void ClickFrame::onSetMouseEvent()
{
  wxPoint point = wxGetMousePosition();

  ostringstream data;
  data  << "lclick (" << point.x << ", " << point.y << ", 100, 1)\n";
    
  text_->AppendText(data.str().c_str());
}

void ClickFrame::onModeChange()
{
  menuFile_->Enable(MENU_SAVE, false);
  menuFile_->Enable(MENU_LOAD, false);
  label_ = "";
  //cout << "mode check " << running_->IsChecked() << endl;
  if (!running_->IsChecked()) return;
  prepareRun();    
  lineNumber_ = 0;
  xOffset_ = 0;
  yOffset_ = 0;
  while(running_->IsChecked()){
    pause();
    string command, arguments;
    ostringstream lineNumber;
    lineNumber << "Linenumber: " << lineNumber_;
    lineNumberText_->SetLabel(lineNumber.str().c_str());
    splitString(program_[lineNumber_], command, arguments);
    if (program_[lineNumber_].find(":") != string::npos){
      label_ = program_[lineNumber_];
    }
    if (command=="mdelay") mDelay(arguments);
    else if (command=="lclick") lClick(arguments);  
    else if (command=="mclick") mClick(arguments);  
    else if (command=="rclick") rClick(arguments);  
    else if (command=="goto")   doGoto(arguments);  
    else if (command=="offset") offset(arguments);  
    else if (command=="gosub") goSubroutine(arguments);
    else if (command=="return") returnFunction(arguments);
    lineNumber_++;
    if ((unsigned int)lineNumber_ == program_.size()){
      running_->SetValue(false);
    }
  }
  menuFile_->Enable(MENU_SAVE, true);
  menuFile_->Enable(MENU_LOAD, true);

}

void ClickFrame::splitString(const string &input, string &command, string &arguments)
{
  string::size_type a, b;
  
  a = input.find("(");
  b = input.find(")");
  
  command = input.substr(0, a);
  arguments = input.substr(a+1,b-a-1);
}

void ClickFrame::stripString(string &data)
{
  for (unsigned int i=0; i<data.size(); i++){
    if (data[i] == ' ' || data[i] == '\t') {
      data.erase(i,1);
      i--;
    }
  }
}


void ClickFrame::prepareRun()
{
  program_.clear();
  string programData;
  
  unsigned int programSize = (unsigned int)text_->GetNumberOfLines();
  
  for (unsigned int i=0; i<programSize; i++){
    string d = text_->GetLineText(i).c_str();
    stripString(d);
    program_.push_back(d);
  }
  
  lineNumber_ = 0;
}

void ClickFrame::pause()
{
  if (pause_->IsChecked()){
    while (pause_->IsChecked()){
      wxUsleep(200);
      wxGetApp().Yield(false);
    }
    wxSleep(5);
  }
}

BEGIN_EVENT_TABLE(ClickFrame, wxFrame)
  EVT_TEXT_ENTER(ENTER_BOX, ClickFrame::onSetMouseEvent)
  EVT_CHECKBOX(MODE, ClickFrame::onModeChange)
  EVT_BUTTON(HELP_BUTTON, ClickFrame::onHelp)
  EVT_MENU(MENU_SAVE, ClickFrame::onSave)
  EVT_MENU(MENU_LOAD, ClickFrame::onLoad)
END_EVENT_TABLE()


/*********************************** COMMANDS *********************************/
void ClickFrame::mDelay(const string &arguments)
{
  istringstream data(arguments);
  int delay;
  data >> delay;
  while (delay != 0 && running_->IsChecked()){
    int delaySlice;
    if (delay >= 200) delaySlice = 200;
    else delaySlice = delay;
      
    ostringstream status;
    status << label_ <<" mdelay("<<delay<<")";
    status_->SetLabel(status.str().c_str());
    wxUsleep(delaySlice);
    delay -= delaySlice;
    wxGetApp().Yield(false);
    pause();
  }
  status_->SetLabel("");
}

void ClickFrame::lClick(const string &arguments)
{
  istringstream data(arguments);
  int x,y, delay, restore;
  char a;
  data >> x >> a>> y >> a >> delay >> a >> restore;

  wxPoint point;  
  if (restore != 0){
    point = wxGetMousePosition();
  }
  
	XTestFakeMotionEvent(display_, -1, x+xOffset_, y+yOffset_, CurrentTime);
	XTestFakeButtonEvent(display_, 1, True, delay);
	XTestFakeButtonEvent(display_, 1, False, CurrentTime);
  if (restore != 0){
  	XTestFakeMotionEvent(display_, -1, point.x, point.y, CurrentTime);  
  }  
  XFlush(display_);
}

void ClickFrame::mClick(const string &arguments)
{
  istringstream data(arguments);
  int x,y, delay, restore;
  char a;
  data >> x >> a>> y >> a >> delay >> a >> restore;

  wxPoint point;  
  if (restore != 0){
    point = wxGetMousePosition();
  }
  
	XTestFakeMotionEvent(display_, -1, x+xOffset_, y+yOffset_, CurrentTime);
	XTestFakeButtonEvent(display_, 2, True, delay);
	XTestFakeButtonEvent(display_, 2, False, CurrentTime);
  if (restore != 0){
  	XTestFakeMotionEvent(display_, -1, point.x, point.y, CurrentTime);  
  }  
  XFlush(display_);
}


void ClickFrame::rClick(const string &arguments)
{
  istringstream data(arguments);
  int x,y, delay, restore;
  char a;
  data >> x >> a>> y >> a >> delay >> a >> restore;
  
  wxPoint point;  
  if (restore != 0){
    point = wxGetMousePosition();
  }

	XTestFakeMotionEvent(display_, -1, x+xOffset_, y+yOffset_, CurrentTime);
	XTestFakeButtonEvent(display_, 3, True, delay);
	XTestFakeButtonEvent(display_, 3, False, CurrentTime);
  if (restore != 0){
  	XTestFakeMotionEvent(display_, -1, point.x, point.y, CurrentTime);  
  }  
  XFlush(display_);
}

void ClickFrame::doGoto(const string &arguments)
{
  string label = arguments+":";
  for (unsigned int i=0; i<program_.size(); i++){
    if (program_[i].find(label) != string::npos){
      label_ = label;
      lineNumber_ = i;
      return;
    }
  }
  string message = "Could not find label called: "+label;
  wxMessageBox(message.c_str());
}

void ClickFrame::offset(const string &arguments)
{
  istringstream data(arguments);
  char a;
  data >> xOffset_ >> a>> yOffset_;
}

void ClickFrame::goSubroutine(const string &arguments)
{
  instructionPointerStack_.push(lineNumber_);
  doGoto(arguments);
}


void ClickFrame::returnFunction(const string &arguments)
{
  if (instructionPointerStack_.size()==0){
    wxMessageBox(_("Returns without gosub"));
    running_->SetValue(false);
    return;
  }
  lineNumber_ = instructionPointerStack_.top();
  instructionPointerStack_.pop();
}
