/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#ifndef MOD_CLICKIT_H
#define MOD_CLICKIT_H

#include <wx/wx.h>

#include <string>
#include <vector>
#include <stack>


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

using namespace std;

class ClickApp : public wxApp
{
  public:
  bool OnInit();
};

class ClickFrame : public wxFrame
{
public:
  ClickFrame(const wxString name);
  ~ClickFrame();

private:
  void stripString(string &data);
  void prepareRun();
  void splitString(const string &input, string &command, string &arguments);
  void pause();

  //commands
  void mDelay(const string &arguments);
  void lClick(const string &arguments);
  void mClick(const string &arguments);
  void rClick(const string &arguments);
  void doGoto(const string &arguments);
  void offset(const string &arguments);
  void goSubroutine(const string &arguments);
  void returnFunction(const string &arguments);


  //events 
  void onSetMouseEvent();
  void onModeChange();
  void onHelp();
  void onSave();
  void onLoad();


  wxMenuBar *menuBar_;
  wxMenu *menuFile_;
  wxTextCtrl *text_;
  wxCheckBox *running_;
  wxCheckBox *pause_;
  wxStaticText *lineNumberText_, *status_;
  
  wxFileDialog *loadFileDialog_, *saveFileDialog_;
  vector<string> program_;
  int lineNumber_;
  string label_;
  stack<int> instructionPointerStack_;
  int xOffset_, yOffset_;  
  Display *display_;

  DECLARE_EVENT_TABLE();
};

enum {
  ENTER_BOX=2,
  MODE,
  HELP_BUTTON,
  MENU_LOAD,
  MENU_SAVE
};

DECLARE_APP(ClickApp)

#endif
